"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = configureProxyRouter;
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const express_1 = __importDefault(require("express"));
const config_js_1 = require("../config.js");
const utils_js_1 = require("../lib/utils.js");
const normalizePath = (routePath) => {
    if (routePath.endsWith('/') && routePath.length > 1) {
        return routePath.slice(0, -1);
    }
    return routePath;
};
const RESERVED_PATHS = new Set([config_js_1.API_PATH]);
function configureProxyRouter({ port }) {
    const router = express_1.default.Router();
    const redirects = {
        [`${config_js_1.API_PATH}/*`]: {
            port,
            path: config_js_1.API_PATH,
        },
    };
    const configureRedirects = async (input) => {
        const newRedirects = Array.isArray(input) ? input : [input];
        // validation
        newRedirects.forEach((redirect) => {
            if (typeof redirect.port !== 'number') {
                throw new TypeError('Redirect should provide "port"');
            }
            if (typeof redirect.path !== 'undefined' && (typeof redirect.path !== 'string' || redirect.path[0] !== '/')) {
                throw new TypeError('Redirect should provide "path" in string format, starting with "/"');
            }
            if (RESERVED_PATHS.has(redirect.path)) {
                throw new TypeError(`Redirect path "${redirect.path}" is reserved`);
            }
        });
        // register
        newRedirects.forEach((redirect) => {
            const normalizedPath = normalizePath(redirect.path || '/');
            redirects[normalizedPath] = {
                ...redirect,
                path: normalizedPath,
            };
            console.log('Registered redirect:', normalizedPath, 'to port', redirect.port);
        });
        await applyRedirects();
    };
    const removeRedirects = async (paths) => {
        paths.forEach((routePath) => {
            if (!routePath || RESERVED_PATHS.has(routePath)) {
                return;
            }
            const normalizedPath = normalizePath(routePath);
            if (redirects[normalizedPath]) {
                delete redirects[normalizedPath];
                console.log('Removed redirect', normalizedPath);
            }
        });
        await applyRedirects();
    };
    const applyRedirects = async () => {
        // apply to the redirects file
        const contents = Object.values(redirects)
            .reduce((res, redirect) => {
            if (redirect.path !== '/') {
                // for non default path add both /path and /path/
                res.push(`${redirect.path} http://localhost:${redirect.port} 200!`);
            }
            res.push(`${path_1.default.posix.join(redirect.path, '*')} http://localhost:${redirect.port}/:splat 200!`);
            return res;
        }, [])
            .join('\n');
        const isFileExists = await promises_1.default
            .access('_redirects')
            .then(() => true)
            .catch(() => false);
        await promises_1.default.writeFile('_redirects', contents);
        if (!isFileExists) {
            // override file once again since file wasn't exists and cli wasn't watching for it
            await (0, utils_js_1.delay)(100);
            await promises_1.default.writeFile('_redirects', contents);
        }
    };
    router.post('/', express_1.default.json(), async (req, res) => {
        try {
            const redirect = Array.isArray(req.body)
                ? req.body
                : {
                    port: req.body.port,
                    path: req.body.path,
                };
            await configureRedirects(redirect);
            res.status(200).send();
        }
        catch (error) {
            console.error('Encountered error while configuring ports', error);
            res.status(500).send({ error });
        }
    });
    router.delete('/', express_1.default.json(), async (req, res) => {
        try {
            const paths = Array.isArray(req.body) ? req.body : [req.body];
            await removeRedirects(paths);
            res.status(200).send();
        }
        catch (error) {
            console.error('Encountered error while removing port configuration', error);
            res.status(500).send({ error });
        }
    });
    return router;
}
