"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = runRouter;
const path_1 = __importDefault(require("path"));
const process_1 = __importDefault(require("process"));
const express_1 = __importDefault(require("express"));
const merge_stream_1 = __importDefault(require("merge-stream"));
const config_js_1 = require("../config.js");
// global since every process gets the incremental PID
const commandsMap = {};
function runRouter({ config, onProcessCreated }) {
    const router = express_1.default.Router();
    router.get('/', (req, res) => {
        const runningCommands = Object.values(commandsMap).filter((command) => command.exitCode === null);
        res.status(200).send(runningCommands.map(commandToOutput));
    });
    router.get('/:commandId', (req, res) => {
        const command = commandsMap[req.params.commandId];
        if (!command) {
            return res.status(404).end();
        }
        res.status(200).send(commandToOutput(command));
    });
    router.delete('/:commandId', (req, res) => {
        const { commandId } = req.params;
        const command = commandsMap[commandId];
        if (!command) {
            return res.status(404).end();
        }
        if (command.exitCode === null) {
            console.log('Stopping command', command.command, 'with PID', command.pid);
            command.kill();
        }
        delete commandsMap[commandId];
        res.status(200).send(commandToOutput(command));
    });
    router.delete('/', (req, res) => {
        Object.values(commandsMap).forEach((command) => {
            if (command.exitCode === null) {
                console.log('Stopping command', command.command, 'with PID', command.pid);
                command.kill();
                delete commandsMap[command.pid];
            }
        });
        res.status(200).end();
    });
    router.post('/', express_1.default.json(), async (req, res) => {
        try {
            const { command, args, detach, timeout, env } = req.body;
            const abortController = new AbortController();
            if (typeof timeout === 'number') {
                setTimeout(() => {
                    abortController.abort();
                }, timeout);
            }
            const commandChild = config.utils.run(command, args, {
                env,
                signal: abortController === null || abortController === void 0 ? void 0 : abortController.signal,
                cwd: path_1.default.join(process_1.default.cwd(), config_js_1.DEFAULT_APP_FOLDER),
            });
            commandChild.command = [command, ...(args !== null && args !== void 0 ? args : [])].join(' ');
            const commandId = commandChild.pid;
            commandsMap[commandId] = commandChild;
            commandChild.on('close', () => {
                console.log('Command has stopped', commandId);
            });
            console.log('Running command', commandChild.command);
            processStream(commandChild);
            onProcessCreated === null || onProcessCreated === void 0 ? void 0 : onProcessCreated(commandChild);
            if (detach) {
                // when detached, make sure to add that catcher so it wont cause unhandled exception
                // eslint-disable-next-line promise/prefer-await-to-then, promise/prefer-await-to-callbacks
                commandChild.catch((error) => {
                    console.error('Command errorer', error);
                });
            }
            const result = detach ? { id: commandId } : processedCommandToOutput(await commandChild);
            res.status(200).send(result);
        }
        catch (error) {
            console.error('Error running command', error);
            res.status(500).send(processedCommandErrorToOutput(error));
        }
    });
    return router;
}
const commandToOutput = (command) => ({
    id: command.pid,
    command: command.command,
    exitCode: command.exitCode,
    stdout: command.stdoutStr,
    stderr: command.stderrStr,
    all: command.allStr,
    killed: command.killed,
});
const processedCommandToOutput = (commandObject) => {
    const { command, exitCode, stdout, stderr, all, killed } = commandObject;
    return {
        command,
        exitCode,
        stdout,
        stderr,
        all,
        killed,
    };
};
const processedCommandErrorToOutput = (commandError) => {
    const { originalMessage, code } = commandError;
    return {
        ...processedCommandToOutput(commandError),
        error: originalMessage,
        errorCode: code,
    };
};
const processStream = (cmd) => {
    const command = cmd;
    command.allStr = '';
    command.all = (0, merge_stream_1.default)();
    command.all.on('data', (chunk) => {
        addToStringBuffer(command, 'allStr', chunk.toString());
    });
    ['stdout', 'stderr'].forEach((key) => {
        const strKey = `${key}Str`;
        command[strKey] = '';
        command.all.add(command[key]);
        command[key].on('data', (chunk) => {
            addToStringBuffer(command, strKey, chunk.toString());
        });
    });
};
const addToStringBuffer = (cmd, bufferName, contents) => {
    const command = cmd;
    command[bufferName] += contents;
    if (command[bufferName].length > config_js_1.MAX_BUFFER_LENGTH) {
        command[bufferName] = command[bufferName].slice(command[bufferName].length - config_js_1.MAX_BUFFER_LENGTH);
    }
};
